import 'dart:async';
import 'package:http/http.dart';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:graphql/internal.dart';

typedef HttpResponseHandler = void Function(FetchResult, StreamedResponse);

class GraphQLInterceptHttpCodeLink extends Link {
  HttpResponseHandler httpResponseHandler;

  GraphQLInterceptHttpCodeLink({this.httpResponseHandler})
      : super(
          request: (Operation operation, [NextLink forward]) {
            StreamController<FetchResult> controller;

            Future<void> onListen() async {
              await controller
                  .addStream(forward(operation).map((FetchResult result) {
                if (result != null && result.errors != null) {
                  StreamedResponse response =
                      operation.getContext()["response"];
                  if ((response != null || response.statusCode >= 300) &&
                      httpResponseHandler != null) {
                    httpResponseHandler(result, response);
                  }
                }
                return result;
              }).handleError((error) {
                StreamedResponse response = operation.getContext()["response"];
                if ((response != null || response.statusCode >= 300) &&
                    httpResponseHandler != null) {
                  String message;
                  String errorMessage =
                      error.message != null ? error.message : '';
                  if (!isStringEmpty(errorMessage)) {
                    List msgList = errorMessage.split(':');
                    if (msgList.length >= 2) {
                      message = msgList[1].toString().trim();
                    }
                  }

                  StreamedResponse streamedResponse = StreamedResponse(
                      response.stream, response.statusCode,
                      reasonPhrase: isStringEmpty(message)
                          ? response.reasonPhrase
                          : message);

                  httpResponseHandler(null, streamedResponse);
                }
                throw error;
              }));

              await controller.close();
            }

            controller = StreamController<FetchResult>(onListen: onListen);

            return controller.stream;
          },
        );

  static bool isStringEmpty(String value) {
    return value == null || value.length <= 0;
  }
}
